#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <mpi.h>

int monte_carlo_pi(int tries)
{
  int hits = 0;
  int i;

  //srand(time(NULL));
  //double x = (double) rand() / RAND_MAX;

  int rank;
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);

  srand48(time(NULL) + rank);

  for (i = 0; i < tries; i++) {
    double x = drand48();
    double y = drand48();
    if (x*x + y*y < 1)
      hits += 1;
  }

  return hits;
}

int main(int argc, char **argv)
{
  MPI_Init(&argc, &argv);
  int rank, size;
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);

  int tries = 100000000;
  int localtries = tries / size;
  if (rank == 0)
    localtries = tries - localtries * (size - 1);

  int localhits = monte_carlo_pi(localtries);
  int hits;
  MPI_Reduce(&localhits, &hits, 1, MPI_INT, MPI_SUM, 0, MPI_COMM_WORLD);

  if (rank == 0) {
    double pi = ((double) hits) / tries * 4.0;
    printf("%f\n", pi);
  }

  MPI_Finalize();
}
